@@ -1,16 +1,14 @@
+import struct
 from gzip import GzipFile
 from io import BytesIO
-import re
-import struct
 
 from scrapy.utils.decorators import deprecated
 
 
-# - Python>=3.5 GzipFile's read() has issues returning leftover
-#   uncompressed data when input is corrupted
-#   (regression or bug-fix compared to Python 3.4)
+# - GzipFile's read() has issues returning leftover uncompressed data when
+#   input is corrupted
 # - read1(), which fetches data before raising EOFError on next call
-#   works here but is only available from Python>=3.3
+#   works here
 @deprecated('GzipFile.read1')
 def read1(gzf, size=-1):
     return gzf.read1(size)
@@ -43,17 +41,5 @@ def gunzip(data):
     return b''.join(output_list)
 
 
-_is_gzipped = re.compile(br'^application/(x-)?gzip\b', re.I).search
-_is_octetstream = re.compile(br'^(application|binary)/octet-stream\b', re.I).search
-
-
-@deprecated
-def is_gzipped(response):
-    """Return True if the response is gzipped, or False otherwise"""
-    ctype = response.headers.get('Content-Type', b'')
-    cenc = response.headers.get('Content-Encoding', b'').lower()
-    return _is_gzipped(ctype) or _is_octetstream(ctype) and cenc in (b'gzip', b'x-gzip')
-
-
 def gzip_magic_number(response):
     return response.body[:3] == b'\x1f\x8b\x08'
