@@ -2,16 +2,17 @@
 Some pipelines used for testing
 """
 
-class ZeroDivisionErrorPipeline(object):
+
+class ZeroDivisionErrorPipeline:
 
     def open_spider(self, spider):
-        a = 1/0
+        1 / 0
 
     def process_item(self, item, spider):
         return item
 
 
-class ProcessWithZeroDivisionErrorPipiline(object):
+class ProcessWithZeroDivisionErrorPipiline:
 
     def process_item(self, item, spider):
-        1/0
+        1 / 0
